import plotly
import plotly.graph_objects as go
import plotly.subplots

default_layout = dict(
    title=dict(
        font=dict(
            size=20,  # Font size of title
            color="black",  # Font color of title
            family="Arial",  # Font family of title
        ),
        x=0.5,  # Center the title
    ),
    xaxis=dict(
        showline=True,  # Show X-axis line
        mirror=True,
        linecolor="black",  # Color of X-axis line
        linewidth=2,
        ticks="outside",  # Show ticks outside the plot on the X-axis
        ticklen=10,  # Length of X-axis ticks
        tickwidth=2,  # Width of X-axis ticks
        tickcolor="black",  # Color of X-axis ticks
        title_font=dict(size=20),
        tickfont=dict(size=20),
        gridcolor="lightgray",  # Set grid color to a faint color
        gridwidth=0.5,  # Set grid line width to a thin value
        showspikes=True,  # Enable vertical spikeline
        spikemode="across",  # Show spike line across all subplots/traces
        spikesnap="cursor",  # Snap spike to cursor position
    ),
    yaxis=dict(
        showline=True,  # Show X-axis line
        mirror=True,
        linecolor="black",  # Color of X-axis line
        linewidth=2,
        ticks="outside",  # Show ticks outside the plot on the X-axis
        ticklen=10,  # Length of X-axis ticks
        tickwidth=2,  # Width of X-axis ticks
        tickcolor="black",  # Color of X-axis ticks
        title_font=dict(size=20),
        tickfont=dict(size=20),
        gridcolor="lightgray",  # Set grid color to a faint color
        gridwidth=0.5,  # Set grid line width to a thin value
    ),
    yaxis2=dict(  # Y-axis for the error bar subplot
        title_font=dict(size=20, family="Arial"),
        showline=True,
        linecolor="black",
        linewidth=2,
        mirror=True,
        ticks="outside",
        tickfont=dict(size=20),
        tickcolor="black",
        gridcolor="lightgray",  # Set grid color to a faint color
        gridwidth=0.5,
    ),
    legend=dict(
        font=dict(
            size=15,  # Font size of legend
            color="black",  # Font color of legend
            family="Arial",  # Font family of legend
        ),
    ),
    plot_bgcolor="rgba(0,0,0,0)",
    hovermode="x unified",  # Show one tooltip box for all traces at a given x
    spikedistance=-1,  # Always show spikes regardless of distance
)


class Figure(plotly.graph_objects.Figure):
    def __init__(
        self, data=None, layout={}, frames=None, skip_invalid=False, **kwargs
    ):
        layout.update(default_layout)
        super(Figure, self).__init__(
            data, layout, frames, skip_invalid, **kwargs
        )

    def add_plot(self, x=[], y=[], name=None):
        self.add_trace(go.Scatter(x=x, y=y, name=name))


def make_subplots(
    rows=1,
    cols=1,
    shared_xaxes=False,
    shared_yaxes=False,
    start_cell="top-left",
    print_grid=False,
    horizontal_spacing=None,
    vertical_spacing=None,
    subplot_titles=None,
    column_widths=None,
    row_heights=None,
    specs=None,
    insets=None,
    column_titles=None,
    row_titles=None,
    x_title=None,
    y_title=None,
    figure=None,
    **kwargs,
) -> go.Figure:
    """
    Makes a subplot with custom Figure object as input
    """
    figure = figure if figure else Figure()
    return plotly.subplots.make_subplots(
        rows=rows,
        cols=cols,
        shared_xaxes=shared_xaxes,
        shared_yaxes=shared_yaxes,
        start_cell=start_cell,
        print_grid=print_grid,
        horizontal_spacing=horizontal_spacing,
        vertical_spacing=vertical_spacing,
        subplot_titles=subplot_titles,
        column_widths=column_widths,
        row_heights=row_heights,
        specs=specs,
        insets=insets,
        column_titles=column_titles,
        row_titles=row_titles,
        x_title=x_title,
        y_title=y_title,
        figure=figure,
        **kwargs,
    )


def make_errorplot(
    xaxis_title="X-axis title",
    yaxis1_title="Y-axis title",
    yaxis2_title="Error",
):
    """
    Make error plot
    """
    # Create a subplot layout with 2 rows and shared x-axis
    fig = make_subplots(
        rows=2,
        cols=1,
        shared_xaxes=True,
        row_heights=[0.7, 0.3],  # Adjust relative heights
        vertical_spacing=0.03,  # Space between subplots
    )
    fig.update_layout(
        dict(
            xaxis=dict(
                showticklabels=False,
                ticks="",
            ),
            yaxis=dict(
                title=yaxis1_title,
                title_font=dict(size=20, family="Arial"),
            ),
            xaxis2=dict(  # X-axis for the bottom subplot (shared x-axis properties)
                title=xaxis_title,
                title_font=dict(size=20, family="Arial"),
                showline=True,
                linecolor="black",
                linewidth=2,
                mirror=True,
                ticks="outside",
                ticklen=10,
                tickwidth=2,
                tickfont=dict(size=20),
                tickcolor="black",
                gridcolor="lightgray",  # Set grid color to a faint color
                gridwidth=0.5,
            ),
            yaxis2=dict(  # Y-axis for the error bar subplot
                title=yaxis2_title,
                title_font=dict(size=20, family="Arial"),
                showline=True,
                linecolor="black",
                linewidth=2,
                mirror=True,
                ticks="outside",
                ticklen=10,
                tickwidth=2,
                tickfont=dict(size=20),
                tickcolor="black",
                gridcolor="lightgray",  # Set grid color to a faint color
                gridwidth=0.5,
            ),
        )
    )

    return fig



import plotly.graph_objects as go


class Scatter(go.Scatter):
    def __init__(
        self,
        arg=None,
        alignmentgroup=None,
        cliponaxis=None,
        connectgaps=None,
        customdata=None,
        customdatasrc=None,
        dx=None,
        dy=None,
        error_x=None,
        error_y=None,
        fill=None,
        fillcolor=None,
        fillgradient=None,
        fillpattern=None,
        groupnorm=None,
        hoverinfo=None,
        hoverinfosrc=None,
        hoverlabel=None,
        hoveron=None,
        hovertemplate=None,
        hovertemplatesrc=None,
        hovertext=None,
        hovertextsrc=None,
        ids=None,
        idssrc=None,
        legend=None,
        legendgroup=None,
        legendgrouptitle=None,
        legendrank=None,
        legendwidth=None,
        line=None,
        marker=None,
        meta=None,
        metasrc=None,
        mode=None,
        name=None,
        offsetgroup=None,
        opacity=None,
        orientation=None,
        selected=None,
        selectedpoints=None,
        showlegend=None,
        stackgaps=None,
        stackgroup=None,
        stream=None,
        text=None,
        textfont=None,
        textposition=None,
        textpositionsrc=None,
        textsrc=None,
        texttemplate=None,
        texttemplatesrc=None,
        uid=None,
        uirevision=None,
        unselected=None,
        visible=None,
        x=None,
        x0=None,
        xaxis=None,
        xcalendar=None,
        xhoverformat=None,
        xperiod=None,
        xperiod0=None,
        xperiodalignment=None,
        xsrc=None,
        y=None,
        y0=None,
        yaxis=None,
        ycalendar=None,
        yhoverformat=None,
        yperiod=None,
        yperiod0=None,
        yperiodalignment=None,
        ysrc=None,
        zorder=None,
        **kwargs,
    ):
        line = line if line else dict(width=2)
        super().__init__(
            arg,
            alignmentgroup,
            cliponaxis,
            connectgaps,
            customdata,
            customdatasrc,
            dx,
            dy,
            error_x,
            error_y,
            fill,
            fillcolor,
            fillgradient,
            fillpattern,
            groupnorm,
            hoverinfo,
            hoverinfosrc,
            hoverlabel,
            hoveron,
            hovertemplate,
            hovertemplatesrc,
            hovertext,
            hovertextsrc,
            ids,
            idssrc,
            legend,
            legendgroup,
            legendgrouptitle,
            legendrank,
            legendwidth,
            line,
            marker,
            meta,
            metasrc,
            mode,
            name,
            offsetgroup,
            opacity,
            orientation,
            selected,
            selectedpoints,
            showlegend,
            stackgaps,
            stackgroup,
            stream,
            text,
            textfont,
            textposition,
            textpositionsrc,
            textsrc,
            texttemplate,
            texttemplatesrc,
            uid,
            uirevision,
            unselected,
            visible,
            x,
            x0,
            xaxis,
            xcalendar,
            xhoverformat,
            xperiod,
            xperiod0,
            xperiodalignment,
            xsrc,
            y,
            y0,
            yaxis,
            ycalendar,
            yhoverformat,
            yperiod,
            yperiod0,
            yperiodalignment,
            ysrc,
            zorder,
            **kwargs,
        )
